import random
import sys
from time import sleep

import boto3

from phoenix_letter.common.arguments import parse_arguments
from phoenix_letter.common.credentials import get_credentials
from phoenix_letter.common.enums import ReasonStopEnum


def main(args=None):
    args = parse_arguments(args)

    if args.input_keys:
        aws_access_key, aws_secret_key = get_credentials()
    else:
        aws_access_key, aws_secret_key = (None, None)

    sqs_client = boto3.client(
        "sqs",
        region_name=args.region,
        aws_access_key_id=aws_access_key,
        aws_secret_access_key=aws_secret_key,
    )

    print("Getting Queue URLs")

    source_queue = sqs_client.get_queue_url(QueueName=args.source)
    source_queue_url = source_queue["QueueUrl"]

    destination_queue = sqs_client.get_queue_url(QueueName=args.destination)
    destination_queue_url = destination_queue["QueueUrl"]

    number_of_empty_receives = 0
    total_messages_received = 0

    reason = None

    while True:
        if number_of_empty_receives == int(args.max_empty_receives_count):
            reason = ReasonStopEnum.EMPTY_RECEIVED
            break
        elif 0 < args.max_messages <= total_messages_received:
            reason = ReasonStopEnum.MAX_MESSAGES_RECEIVED
            break

        print("Receiving message...")
        received_response = sqs_client.receive_message(
            QueueUrl=source_queue_url,
            MessageAttributeNames=["All"],
            AttributeNames=["All"],
            MaxNumberOfMessages=args.max_receive_messages,
        )

        if ("Messages" not in received_response) or (
            len(received_response["Messages"]) == 0
        ):
            print("Queue did not returned messages")

            number_of_empty_receives += 1
            sleep_time = random.randint(500, 2000) / 1000

            print("Sleeping for {} seconds".format(sleep_time))
            sleep(sleep_time)

            continue

        messages_received = len(received_response["Messages"])

        total_messages_received += messages_received

        print("Received {} messages".format(messages_received))

        for message in received_response["Messages"]:
            print("Sending message to '{}'".format(args.destination))

            if "MessageAttributes" in message:
                sqs_client.send_message(
                    QueueUrl=destination_queue_url,
                    MessageBody=message["Body"],
                    MessageAttributes=message["MessageAttributes"],
                )
            else:
                sqs_client.send_message(
                    QueueUrl=destination_queue_url, MessageBody=message["Body"]
                )

            print("Deleting message from '{}'".format(args.source))
            sqs_client.delete_message(
                QueueUrl=source_queue_url, ReceiptHandle=message["ReceiptHandle"]
            )

    if reason == ReasonStopEnum.MAX_MESSAGES_RECEIVED:
        print("Stopping after processing {} messages.".format(total_messages_received))
    else:
        print(
            "Giving up after {} empty receives from the source queue.".format(
                number_of_empty_receives
            )
        )

    return reason


if __name__ == "__main__":  # pragma: no cover
    response = main(sys.argv[1:])
    print("Stop Reason: {}".format(response.name))
    sys.exit(0)



from argparse import ArgumentParser


def parse_arguments(args):
    parser = ArgumentParser()
    parser.add_argument(
        "--src",
        dest="source",
        required=True,
        help="Source SQS Queue Name",
        metavar="SOURCE_QUEUE",
    )

    parser.add_argument(
        "--dst",
        dest="destination",
        required=True,
        help="Destination SQS Queue Name",
        metavar="DESTINATION_QUEUE",
    )

    parser.add_argument(
        "--aws-keys",
        dest="input_keys",
        help="Flag that indicates you want to enter custom AWS keys.",
        action="store_true",
    )

    parser.add_argument(
        "--region",
        dest="region",
        default="us-east-1",
        required=True,
        help="AWS Region",
        metavar="REGION",
    )

    parser.add_argument(
        "--empty-receive",
        dest="max_empty_receives_count",
        default=10,
        help="Max number of empty receives before giving up",
        metavar="EMPTY_RECEIVE",
    )

    parser.add_argument(
        "--max",
        dest="max_messages",
        default=0,
        type=int,
        help="Max number of messages to process from the source queue.",
        metavar="N",
    )

    parser.add_argument(
        "--max-per-request",
        dest="max_receive_messages",
        default=10,
        type=int,
        choices=range(1, 11),
        help="Max number of messages to received from the source queue per request (this will be pass "
        "in the MaxNumberOfMessages param). Default: 10 (AWS API max limit)",
        metavar="N",
    )

    return parser.parse_args(args)



import codecs
import os
import re
from glob import glob
from os.path import basename, splitext

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), "r") as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def parse_readme():
    with open("README.md") as readme_file:
        return readme_file.read()


def parse_requirements_file(req):
    with open(req) as file:
        lines = file.readlines()
        reqs = [item.split("==")[0] for item in lines]
        return [item for item in reqs if item.startswith("-") is False]


setup(
    name="phoenix_letter",
    version=find_version("src", "phoenix_letter", "version.py"),
    url="https://github.com/renanvieira/phoenix-letter",
    license="MIT",
    author="Renan Vieira",
    author_email="me@renanvieira.net",
    package_dir={"phoenix_letter": "src/phoenix_letter"},
    packages=find_packages("src"),
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    description="CLI to move messages from a AWS SQS Queue to another",
    long_description=parse_readme(),
    long_description_content_type="text/markdown",
    keywords=["AWS", "SQS", "Queue", "DLQ", "Dead", "Letter", "Queue"],
    python_requires=">=2.7",
    zip_safe=False,
    entry_points={"console_scripts": ["phoenix_letter=phoenix_letter.main:main"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    install_requires=["boto3", "six", "mock"],
    test_suite="tests/",
    tests_require=["nose", "moto", "coverage"],
)
